# Reference
<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

**Classes**

_Public Classes_

* [`znapzend`](#znapzend): Main class for managing the installation and configuration of the znapzend service.
* [`znapzend::imports`](#znapzendimports): Class for collecting and created znapzend backup plans to import onto datasets.

_Private Classes_

* `znapzend::config`: This class is called from znapzend for service config.
* `znapzend::install`: This class is called from the main znapzend class for install.
* `znapzend::prereqs`: This class is called from the main znapzend class for installing prerequisites.
* `znapzend::repos`: This class is called from the main znapzend class for adding needed repos.
* `znapzend::service`: This class is meant to be called from znapzend to manage the znapzend service.

**Defined types**

* [`znapzend::import`](#znapzendimport): Define znapzend::import ===========================  Defined type for a backup plan for a  zfs dataset to be imported by znapzendzetup. Note:

## Classes

### znapzend

Class: znapzend
===========================

Main class that includes all other classes for the znapzend module.

#### Parameters

The following parameters are available in the `znapzend` class.

##### `gcc_packages`

Data type: `Array`

Specifies what packages are installed for the build requirements of znapzend for providing gcc/make.

Default value: [ 'gcc', 'gcc-c++', ]

##### `manage_epel`

Data type: `Boolean`

Specifies whether or not the module should manage the EPEL repository.

Default value: `true`

##### `manage_gcc`

Data type: `Boolean`

Specifies whether or not the module should manage the installation of gcc.

Default value: `true`

##### `manage_mbuffer`

Data type: `Boolean`

Specifies whether or not the module should manage the installation of mbuffer.

Default value: `true`

##### `manage_perl`

Data type: `Boolean`

Specifies whether on not the module should manage the installation of the perl-core rpm.

Default value: `true`

##### `manage_prereqs`

Data type: `Boolean`

Specifies whether or not the module should manage any of the components of the preqrequites for building znapzend.

Default value: `true`

##### `mbuffer_packages`

Data type: `Array`

Specifies what package(s) should be installed on the system to provide mbuffer support for znapzend to use.

Default value: [ 'mbuffer', ]

##### `perl_packages`

Data type: `Array`

Specifies what perl RPM(s) should be installed on the system for building and running znapzend.

Default value: [ 'perl-core', ]

##### `plan_confdir`

Data type: `Stdlib::Unixpath`

Specifies where the znapzend backup plan configuration files will be located.

Default value: '/etc/znapzend/configs'

##### `plan_confdir_purge`

Data type: `Boolean`

Specifies whether or not the znapzend configuration directories should be purged of files not managed by puppet.

Default value: `true`

##### `plan_confdir_setup`

Data type: `Array[Stdlib::Unixpath]`

An array of directories that puppet should create on the system for where the znapzend backup plans will be stored.

Default value: [ '/etc/znapzend', '/etc/znapzend/configs', ]

##### `plan_conffile_mode`

Data type: `String`

The mode that should be used for the znapzend backup plan configuration files.

Default value: '0644'

##### `plan_conffile_template`

Data type: `String`

The name of the template to use for the znapzend backup plan configuration files.

Default value: 'znapzend/plan_conffile_template.erb'

##### `service_enable`

Data type: `Boolean`

Whether to enable the znapzend service at boot.

Default value: `true`

##### `service_ensure`

Data type: `Enum['running', 'stopped']`

Whether the znapzend service should be running.

Default value: 'running'

##### `service_name`

Data type: `String`

Specifies the name of the service to manage.

Default value: 'znapzend'

##### `service_options`

Data type: `String`

Specifies options placed in /etc/default/znapzend for use by the znapzend service.

Default value: ''

##### `service_options_template`

Data type: `String`

Specifies the name of the template to use for /etc/default/znapzend.

Default value: 'znapzend/znapzend.default.erb'

##### `service_systemd_afters`

Data type: `Array`

Specifes what systemd services should be started up prior to the znapzend.service.

Default value: [ 'zfs-import-cache.service', 'zfs-import-scan.service', ]

##### `service_systemd_template`

Data type: `String`

Specifies the name of the template to use for the /etc/systemd/system/znapzend.service file.

Default value: 'znapzend/znapzend.service.erb'

##### `service_sysv_template`

Data type: `String`

Specifies the name of the template to use for the /etc/init.d/znapzend file.

Default value: 'znapzend/znapzend.sysv.erb'

##### `znapzend_download_location`

Data type: `Stdlib::Unixpath`

Specifies where the znapzend archive resource should download the tar.gz file to for later extraction.

Default value: '/tmp'

##### `znapzend_package_version`

Data type: `String`

Specifies the version of znapzend to download and install.

Default value: '0.19.1'

##### `znapzend_package_extractpath`

Data type: `Stdlib::Unixpath`

Specifies where the znapzend archive resource should extract the tar.gz file to for building

Default value: '/usr/local/src'

##### `znapzend_package_url`

Data type: `Stdlib::Httpurl`

Specifies the URL of where to download the znapzend tar.gz file. By default, this uses the znapzend_package_version version.

Default value: "https://github.com/oetiker/znapzend/releases/download/v${znapzend_package_version}/znapzend-${znapzend_package_version}.tar.gz"

##### `znapzend_install_prefix`

Data type: `Stdlib::Unixpath`

Specifies the directory to which znapzend should be installed.

Default value: "/opt/znapzend-${znapzend_package_version}"

##### `znapzend_installed_binaries`

Data type: `Array`

Specifies a list of znapzend programs to link.

Default value: [ 'znapzend', 'znapzendzetup', 'znapzendztatz', ]

##### `znapzend_linkpath`

Data type: `Stdlib::Unixpath`

Specifies where the znapzend program links should be created.

Default value: '/usr/local/bin'

### znapzend::imports

Class znapzend::imports
===========================

Class for collecting and created znapzend backup plans to import onto datasets. Data is either passed via the imports plans parameter or via hiera.

#### Parameters

The following parameters are available in the `znapzend::imports` class.

##### `plans`

Data type: `Hash`

Hash of plans for which each will result in creation of a config file which is then imported into the properties of the zfs file system for use by znapzend. The title of each plan needs to be the zfs file system name in the form of poolname/filesystem/filesystem.

Default value: {}

## Defined types

### znapzend::import

Define znapzend::import
===========================

Defined type for a backup plan for a  zfs dataset to be imported by znapzendzetup. Note: the title of the defined type should match the zfs file system name not the mountpoint.

#### Parameters

The following parameters are available in the `znapzend::import` defined type.

##### `options`

Data type: `Hash`

A hash of znapzend options to be imported into the file system.

